#!/usr/bin/python

# Resources:
# http://wiki.maemo.org/PyMaemo/Using_Location_API
# http://blog.lucanatali.it/2006/12/nmea-checksum-in-python.html
# http://gpsd.berlios.de/gpsfake.html

import location, gobject
import pty, termios, os, time
from operator import xor
        
def on_error(control, error, data):
    print "location error: %d... quitting" % error
    data.quit()
 
def on_changed(device, data):
    if not device:
        return
    if device.fix:
        if device.fix[1] & location.GPS_DEVICE_LATLONG_SET:
            print "lat = %f, lon = %f" % device.fix[4:6]
            lat = abs(device.fix[4])
            lon = abs(device.fix[5])
            latdm = "%02.0f%09.6f" % (lat//1, lat%1*60.0)              
            londm = "%03.0f%09.6f" % (lon//1, lon%1*60.0) 
            if device.fix[4] < 0.0:
                ns = "S"
            else:
                ns = "N"
            if device.fix[5] < 0.0:
                we = "W"
            else:
                we = "E"
        else:
            latdm = ""
            londm = ""
            ns = ""
            we = ""
        if device.fix[1] & location.GPS_DEVICE_ALTITUDE_SET:
            alt = "%f" % (device.fix[7])
        else:
            alt = ""
        if device.fix[1] & location.GPS_DEVICE_TIME_SET:
            gpstime = time.gmtime(device.fix[2])
            utctime = time.strftime("%H%M%S", gpstime)
            utcdate = time.strftime("%d%m%y", gpstime)
        else:
            utctime = ""
            utcdate = ""
        if device.fix[1] & location.GPS_DEVICE_TRACK_SET:
            course_deg = "%.1f" % (device.fix[9])
        else:
            course_deg = ""
        if device.fix[1] & location.GPS_DEVICE_SPEED_SET:
            speed_knots = "%.1f" % (device.fix[11]/1.852)
        else:
            speed_knots = "" 
        print "Satellites in view: %d, in use: %d" % (device.satellites_in_view, device.satellites_in_use)
	if device.fix[0] == 3: #LOCATION_GPS_DEVICE_MODE_3D
		fixmode = "3"
		status = "A"
                quality = "1"
	elif device.fix[0] == 2: #LOCATION_GPS_DEVICE_MODE_2D
		fixmode = "2"
		status = "A"
                quality = "1"
	else: #LOCATION_GPS_DEVICE_MODE_NOT_SEEN, LOCATION_GPS_DEVICE_MODE_NO_FIX
		fixmode = "1"
		status = "V"
                quality = "0"
        #print location.GPSDevice.satellites # Array of satellites
        # data.stop() commented out to allow continuous loop for a reliable fix - press ctrl c to break the loop

        # RMC - Recommend Minimum Specific GPS Data (position and time)
        sentence = "$GPRMC,%s,%s,%s,%s,%s,%s,%s,%s,%s,,,A*" % (utctime, status, latdm, ns, londm, we, speed_knots, course_deg, utcdate)
        # checksum is the 8-bit XOR of all characters in the sentence between '$' and '*' (exclusive), as two digit hex value
	checksum = reduce(xor, map(ord, sentence[1:sentence.index('*')]))
        # add the checksum in hex format
	sentence += "%02X"%checksum  + "\r\n"
        print sentence
        os.write(master_fd, sentence)

        # GSV - GPS Satellites in View
        sentence = "$GPGSV,1,1,%02d,,,,,*" % (device.satellites_in_view)
        #sentence = "$GPGSV,1,1,%02d,,,,,,,,,,,,,,,,*" % (device.satellites_in_view)
        checksum = reduce(xor, map(ord, sentence[1:sentence.index('*')]))
        sentence += "%02X"%checksum  + "\r\n"
        print sentence
        os.write(master_fd, sentence)

        # GGA - GPS Fix Data
        sentence = "$GPGGA,%s,%s,%s,%s,%s,%s,%02d,,%s,M,0.0,M,,*" % (utctime, latdm, ns, londm, we, quality, device.satellites_in_use, alt)
        checksum = reduce(xor, map(ord, sentence[1:sentence.index('*')]))
        sentence += "%02X"%checksum  + "\r\n"
        print sentence
        os.write(master_fd, sentence)

        # GPGLL -  Geographic Position Lat/Lon
        sentence = "$GPGLL,%s,%s,%s,%s,%s,%s*" % (latdm, ns, londm, we, utctime, status)
        checksum = reduce(xor, map(ord, sentence[1:sentence.index('*')]))
        sentence += "%02X"%checksum  + "\r\n"
        print sentence
        os.write(master_fd, sentence)
        
        # GPGSA - GPS DOP and Active Satellites
        sentence = "$GPGSA,A,%s,,,,,,,,,,,,,,,*" % (fixmode)
        checksum = reduce(xor, map(ord, sentence[1:sentence.index('*')]))
        sentence += "%02X"%checksum  + "\r\n"
        print sentence
        os.write(master_fd, sentence)
        
        # Map Datum (Garmin proprietary sentence)
        #os.write(master_fd, "$PGRMM,WGS 84*06\r\n")
 
def on_stop(control, data):
    print "quitting"
    data.quit()
 
def start_location(data):
    data.start()
    return False

# find and configure an available pseudo-terminal 
(master_fd, slave_fd) = pty.openpty()
print os.ttyname(slave_fd)
(iflag, oflag, cflag, lflag, ispeed, ospeed, cc) = termios.tcgetattr(slave_fd)
cc[termios.VMIN] = 1
cflag &= ~(termios.PARENB | termios.PARODD | termios.CRTSCTS)
cflag |= termios.CREAD | termios.CLOCAL
iflag = oflag = lflag = 0
iflag &=~ (termios.PARMRK | termios.INPCK)
cflag &=~ (termios.CSIZE | termios.CSTOPB | termios.PARENB | termios.PARODD)
cflag |= termios.CS8
ispeed = ospeed = termios.B4800
termios.tcsetattr(slave_fd, termios.TCSANOW,
                  [iflag, oflag, cflag, lflag, ispeed, ospeed, cc])

loop = gobject.MainLoop()
control = location.GPSDControl.get_default()
device = location.GPSDevice()
#device.reset_last_known() # Resets the last known location to unknown.
control.set_properties(preferred_method=location.METHOD_USER_SELECTED, #GNSS AGNSS
                       preferred_interval=location.INTERVAL_DEFAULT)
 
control.connect("error-verbose", on_error, loop)
device.connect("changed", on_changed, control)
control.connect("gpsd-stopped", on_stop, loop)
 
gobject.idle_add(start_location, control)
 
loop.run()
